// Random.cpp: Test various Dev functions and classes
// Created 10/10/99

// Copyright  2000 David Spector
// All Rights Reserved

#define				VC_EXTRALEAN
#define				WIN32_LEAN_AND_MEAN
#define				WIN32_EXTRA_LEAN
#include "Windows.h"
#include <stdlib.h>
#include <stdarg.h>

char				AppName[] = "Random";
char				Out[1024];
char *				pOut;

//	----------------------------------------------------------------------------
//	Name:			function Box
//
//	Description:	Message box function---given printf-type arguments, display
//					the formatted text in a standard message box. The first
//					character determines the type of message box to be
//					displayed. Return ID of user's button click (IDOK, etc.).
//	----------------------------------------------------------------------------
long				Box(const char * Format, ...)
	{
	va_list 		Args;
	char *			pText;
	UINT			Flags;
	int				Status;
	char *			Text;

	Text = (char *)malloc(9000);
	pText = Text;

	// Format the string
	va_start(Args, Format);
	wvsprintf(pText, Format, Args);
	va_end(Args);

	// Handle different types of message boxes
	if (*pText == '?')
		{
		pText++;
		Flags = MB_ICONQUESTION | MB_YESNO;
		if (*pText == 'n')
			Flags |= MB_DEFBUTTON2;
		if (*pText == 'c')
			Flags = MB_ICONINFORMATION | MB_OKCANCEL;
		pText++;
		}
	else if (*pText == '!')
		{
		Flags = MB_ICONEXCLAMATION | MB_OK;
		pText++;
		}
	else if (*pText == '*')
		{
		Flags = MB_ICONSTOP | MB_OK;
		pText++;
		}
	else
		Flags = MB_ICONINFORMATION | MB_OK;

	//TRACE("Box> \"%s\"", pText);
 	Status = MessageBox(NULL, pText, AppName, Flags);
	//if (!Status)
	//	TRACE("*** Memory is low; cannot display message box");
	free(Text);
	return Status;
    } // End of Box

//	----------------------------------------------------------------------------
//	Name:			function Random, internal function RandomNoInit
//
//	Description:	Given an integer range, return a pseudo-random number r
//					within that range, such that LowerValue <= r < UpperValue.
//	----------------------------------------------------------------------------
static DWORD		RandomNoInit(DWORD LowerValue, DWORD UpperValue)
	{
	static DWORD	table[11] =
		{
		0x233B6180, 0x19833D00, 0x3A79AA80, 0x042C6200, 0x79523B7F, 0x1B26AF00,
		0x49D7547F, 0x393B6400, 0x50AD357F, 0x517DC0FF, 0x119A1E80
		//0x7be9, 0x088a, 0x3d38, 0x746b, 0x2d04, 0x775d,
		//0x53c4, 0x02b2, 0x418f, 0x19db, 0x7851};
		};
	static BOOL		init;
	static long		cur, next;
	long			Count;
	DWORD			CurValue;
	
	if (LowerValue >= UpperValue)
		{
		Box("*Attempt to generate a pseudo-random number in the invalid "
			"range %ld to %ld.", LowerValue, UpperValue);
		return 0;
		}

	// Randomize table once only
	if (!init)
		{
		for (Count = 0; Count < 11; Count++)
			table[Count] += GetTickCount();
		init = TRUE;
		}

	// Keep index within table
	if (cur == 10)
		next = 0;
	else
		next = cur + 1;
	
	// Rotate current table entry left 13 and add it to next table entry
	// (13 is largest prime nr less than bitsize_of_long/2)
	table[next] += ((table[cur] << 13) + (table[cur] >> 19));
	cur = next;
	CurValue = table[next];
	return LowerValue + CurValue % (UpperValue - LowerValue);
	//return table[next] % range;
	} // RandomNoInit
DWORD				Random(DWORD LowerValue, DWORD UpperValue)
	{
	static BOOL		init;
	long			Count;

	// Randomize table once only
	if (!init)
		{
		for (Count = 0; Count < 22; Count++)
			RandomNoInit(0, 1);
		init = TRUE;
		}
	return RandomNoInit(LowerValue, UpperValue);
	} // Random

//	----------------------------------------------------------------------------
//	Name:			function ShowSomeRandomNrs
//
//	Description:	A test of 'Random': Generate and show some pseudo-random nrs
//					between 0 and 0xffffffff.
//	----------------------------------------------------------------------------
VOID				ShowSomeRandomNrs()
	{
	long			i, x;
	char			t[256];
	char *			pt;

	pOut += wsprintf(pOut, "Some pseudo-random numbers:\n");
	pt = t;
	for (i = 0; i < 16; i++)
		{
		x = Random(0, (DWORD)-1);
		pOut += wsprintf(pOut, "%.8lx, ", x);
		}
	} // ShowSomeRandomNrs

//	----------------------------------------------------------------------------
//	Name:			function ShowSomeRandomNrs
//
//	Description:	A test of 'Random': Generate and show a histogram of 1100
//					pseudo-random nrs in the range 0 to the nr of bins (10) to
//					show the flat distribution of the pseudo-random nrs.
//	----------------------------------------------------------------------------
VOID				ShowHistogram()
	{
	// Sample histograms:

	// Trials: 10000
	//TestSW> 0: 979
	//TestSW> 1: 998
	//TestSW> 2: 1022
	//TestSW> 3: 1010
	//TestSW> 4: 953
	//TestSW> 5: 1055
	//TestSW> 6: 1005
	//TestSW> 7: 1011
	//TestSW> 8: 927
	//TestSW> 9: 1040
	//TestSW> Sum: 10000

	// Trials: 11*100000
	//TestSW> 0: 100262
	//TestSW> 1: 100467
	//TestSW> 2: 99805
	//TestSW> 3: 100062
	//TestSW> 4: 99958
	//TestSW> 5: 99961
	//TestSW> 6: 100222
	//TestSW> 7: 100381
	//TestSW> 8: 99681
	//TestSW> 9: 99316
	//TestSW> 10: 99885
	//TestSW> Sum: 1100000

	// Trials: 11*1000000
	//TestSW> 0: 1000593
	//TestSW> 1: 999952
	//TestSW> 2: 999906
	//TestSW> 3: 999211
	//TestSW> 4: 999958
	//TestSW> 5: 998942
	//TestSW> 6: 999584
	//TestSW> 7: 999869
	//TestSW> 8: 1001634
	//TestSW> 9: 1000281
	//TestSW> 10: 1000070
	//TestSW> Sum: 11000000

	// Trials: 11*555555
	//TestSW> 0: 555453
	//TestSW> 1: 555416
	//TestSW> 2: 555622
	//TestSW> 3: 557265
	//TestSW> 4: 555458
	//TestSW> 5: 556011
	//TestSW> 6: 555620
	//TestSW> 7: 554208
	//TestSW> 8: 555815
	//TestSW> 9: 556487
	//TestSW> 10: 553750
	//TestSW> Sum: 6111105

#define				HIST_LEN 10
#define				TRIALS 100000
	long			i;
	long			Hist[HIST_LEN];
	long			Index;
	DWORD			Sum;

	pOut += wsprintf(pOut, "Histogram of %ld pseudo-random nrs:\n", TRIALS);

	for (i = 0; i < HIST_LEN; i++)
		Hist[i] = 0;

	for (i = 0; i < TRIALS; i++)
		{
		Index = Random(0, HIST_LEN);
		Hist[Index] = Hist[Index] + 1;
		}

	Sum = 0;
	for (i = 0; i < HIST_LEN; i++)
		Sum += Hist[i];

	for (i = 0; i < HIST_LEN; i++)
		pOut += wsprintf(pOut, "Bin %ld: %ld nrs, ", i, Hist[i]);
	//pOut += wsprintf(pOut, "\nNr of calls of 'Random': %ld", TRIALS);
	} // ShowHistogram

int APIENTRY WinMain(HINSTANCE, // hInstance,
					 HINSTANCE, // hPrevInstance,
					 LPSTR, //	   lpCmdLine,
					 int) //	   nCmdShow
	{
	pOut = Out;
	Out[0] = 0;

	ShowSomeRandomNrs();

	pOut += wsprintf(pOut, "\n------\n");
	
	ShowHistogram();
	Box("%s", Out);
	return 0;
	} // WinMain

// End of Random.cpp
